import React, { Component } from "react";

/**
 * @class
 * @classdesc This class allows users to input information about their age, sex, weight, height in order to receive health stats.
 */
export class DataEntryForm extends Component {
  /**
   * Constructor that takes in user's input information.
   * @constructor
   * @param {string} age
   * @param {string} sex
   * @param {string} weight 
   * @param {string} wunits - units of weights
   * @param {string} height
   * @param {string} hunits - units of height
   * @param {boolean} errorsex - invalid input on sex
   * @param {boolean} errorhunits - invalid input on units of height
   * @param {boolean} errorwunits - invalid input on units of weight
   
   */
  constructor(props) {
    super(props);
    
    this.state = {
      age: "",
      sex: "",
      weight: "",
      wunits: "",
      height: "",
      hunits: "",
      errorsex: false,
      errorhunits: false,
      errorwunits: false
    };

    this.addweight = this.addweight.bind(this);
    this.addweightUnits = this.addweightUnits.bind(this);
    this.addheight = this.addheight.bind(this);
    this.addheightUnits = this.addheightUnits.bind(this);
    this.addage = this.addage.bind(this);
    this.setsex = this.setsex.bind(this);
    this.enterForm = this.enterForm.bind(this);
  }
  /**
   * User inputs his/her weight to the system.
   * @param {string} w - The users weight as a number.
   */
  addweight(w) {
    this.setState({ ...this.state, ...{ weight: w.target.value } });
  }
  /**
   * User inputs the weight's unit in either lbs or kgs
   * @param {string} unitw - Weight in lbs or kgs.
   */
  addweightUnits(unitw) {
    if (unitw.target.value !== "lb" && unitw.target.value !== "kg") {
      this.setState({ ...this.state, ...{ errorwunits: true } });
    } else {
      this.setState({ ...this.state, ...{ wunits: unitw.target.value } });
      this.setState({ ...this.state, ...{ errorwunits: false } });
    }
  }
  /**
   * User inputs his/her height to the system.
   * @param {string} h - Users height input as a number.
   */
  addheight(h) {
    this.setState({ ...this.state, ...{ height: h.target.value } });
  }
  /**
   * User inputs the height's units as feet and inches or meters and cm
   * @param {string} unith 
   */
  addheightUnits(unith) {
    if (
      unith.target.value !== "cm" &&
      unith.target.value !== "m" &&
      unith.target.value !== "in" &&
      unith.target.value !== "ft"
    ) {
      this.setState({ ...this.state, ...{ errorhunits: true } });
    } else {
      this.setState({ ...this.state, ...{ hunits: unith.target.value } });
      this.setState({ ...this.state, ...{ errorhunits: false } });
    }
  }
  /**
   * User inputs age.
   * @param {string} a 
   */
  addage(a) {
    this.setState({ ...this.state, ...{ age: a.target.value } });
  }
  /**
   * User inputs sex as male, female, other.
   * @param {string} gen 
   */
  setsex(gen) {
    if (
      gen.target.value !== "Male" &&
      gen.target.value !== "male" &&
      gen.target.value !== "Female" &&
      gen.target.value !== "female" &&
      gen.target.value !== "Other" &&
      gen.target.value !== "other"
    ) {
      this.setState({ ...this.state, ...{ errorsex: true } });
    } else {
      this.setState({ ...this.state, ...{ sex: gen.target.value } });
      this.setState({ ...this.state, ...{ errorsex: false } });
    }
  }
  /**
   * This method just prompts users to check if the information they entered is correct.
   */
  enterForm() {
    {
      let confirmation = prompt(
        "Do the following look correct (Y/N)? \n Age is: " +
          this.state.age +
          "\n Sex is: " +
          this.state.sex +
          "\n Height is: " +
          this.state.height +
          " " +
          this.state.hunits +
          "\n Weight is: " +
          this.state.weight +
          " " +
          this.state.wunits
      );
      if (confirmation === "Y" || confirmation === "y") {
        //Store data and proceed
      } else if (confirmation === "N" || confirmation === "n") {
        //Allow user to change information
      } else {
        alert("Please enter Y for Yes or N for No");
        this.enterForm();
      }
    }
  }
  /**
   * Renders the form so user can input the four parameters of data.
   */
  render() {
    return (
      <div className="Form">
        <form onSubmit={this.enterForm}>
          <label htmlFor="Entry">
            Enter Age:
            <input
              className="form-control"
              type="input"
              name="age"
              value={this.state.age}
              placeholder="Age is..."
              onChange={this.addage}
            />
          </label>
          <span key={0}>
            <br />
            <br />
          </span>
          <label htmlFor="Entry">
            Choose Sex:
            <input
              className="form-control"
              type="text"
              name="sex"
              placeholder="Male, Female, or Other..."
              onChange={this.setsex}
            />
            <React.Fragment>
              {this.state.errorsex ? <h1>Invalid entry for sex</h1> : null}
            </React.Fragment>
          </label>
          <span key={1}>
            <br />
            <br />
          </span>
          <label htmlFor="Entry">
            Enter Height and it&apos;s Units:
            <input
              className="form-control"
              type="input"
              name="height"
              placeholder="Height is..."
              onChange={this.addheight}
            />
            <input
              className="form-control"
              type="input"
              name="hunits"
              placeholder="in, ft, cm, or m"
              onChange={this.addheightUnits}
            />
            <React.Fragment>
              {this.state.errorhunits ? (
                <h1>Invalid entry for units of height</h1>
              ) : null}
            </React.Fragment>
          </label>
          <span key={2}>
            <br />
            <br />
          </span>
          <label htmlFor="Entry">
            {" "}
            
          Enter Weight and it&apos;s Units:
           
            <input
              className="form-control"
              type="input"
              name="weight"
              placeholder="Weight is..."
              onChange={this.addweight}
            />
            <input
              className="form-control"
              type="input"
              name="wunits"
              placeholder="lb or kg"
              onChange={this.addweightUnits}
            />
            <React.Fragment>
              {this.state.errorwunits ? (
                <h1>Invalid entry for units of weight</h1>
              ) : null}
            </React.Fragment>
          </label>
          <span key={3}>
            <br />
            <br />
          </span>
          <input type="submit" value="submit" />
        </form>
      </div>
    );
  }
}
