"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var graphql_tools_1 = require("graphql-tools");
var context_1 = require("./context");
var resolvers_1 = require("./resolvers");
var schema_1 = require("./schema");
var graphql_2 = require("graphql");
var subscriptions_transport_ws_1 = require("subscriptions-transport-ws");
/**
 * Makes schema from schema definition (typeDefs) and resolvers (functions that resolve the requested query)
 */
var schema = graphql_tools_1.makeExecutableSchema({ typeDefs: schema_1.typeDefs, resolvers: resolvers_1.resolvers });
/**
 * Processes GraphQL request and returns the result
 *
 * @param args are values needed to execute the GraphQL query. args.source is the query itself,
 * args.operationName is the name of the query, args.variableValues are variables sent with the query,
 * and contextValue is a custom context object that can be passed to the query so the requested
 * resource can authenticate access to the resource.
 * @param res is the express Response object that GraphQL will send the processed response to.
 */
function serveGraphQLRequest(args, res) {
    graphql_1.graphql(__assign({ schema: schema }, args)).then(function (result) {
        res.send(result);
    });
}
exports.serveGraphQLRequest = serveGraphQLRequest;
/**
 * Using the server as the input, this function creates the subscription
 * capabilities using Websocket for GraphQL. This means that time series data
 * can be given to the client (for example the client can subscribe to data, and
 * as the backend recieves the data one at a time, the data can slowly be pushed
 * to the client rather than having to wait for all the data to arrive at the backend
 * before pushing it to the client).
 * ws://localhost:8080/subscriptions
 *
 * A conceptual overview can be found on https://graphql.org/blog/subscriptions-in-graphql-and-relay/
 * @param server http server, usually created from an express instance
 */
function createGraphQLSubscription(server) {
    new subscriptions_transport_ws_1.SubscriptionServer({
        execute: graphql_2.execute,
        subscribe: graphql_2.subscribe,
        schema: schema,
        onConnect: function (params) {
            if (!params.authorization)
                return null;
            var accountId = context_1.Context.verifyToken(params.authorization);
            if (!accountId)
                return;
            var context = new context_1.Context(accountId);
            console.log("ws://onConnect: (authorization) => accountId: " + context.getEtheriumAccountId());
            return context;
        }
    }, {
        server: server,
        path: '/subscriptions'
    });
}
exports.createGraphQLSubscription = createGraphQLSubscription;
//# sourceMappingURL=index.js.map