"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_subscriptions_1 = require("graphql-subscriptions");
/**
 * Resolver for the Query localizedHelloWorld
 * What is a graphql resolver? https://www.apollographql.com/docs/apollo-server/data/data/#resolver-map
 * The schema that corresponds to this is in ../schema/hello-world.graphql
 */
var HW_EVENT_ID = 'helloWorldEvent';
var pubsub = new graphql_subscriptions_1.PubSub();
var counter = 0;
var resolver = {
    Query: {
        localizedHelloWorld: function () {
            return {
                java: "System.out.println('Hello, World');",
                python: "print('Hello, World')",
                cpp: "std::cout << 'Hello, World' << std::endl;",
                javascript: "console.log('Hello, World')"
            };
        }
    },
    Subscription: {
        helloWorldSubscription: {
            subscribe: function () {
                counter = 0;
                var it = pubsub.asyncIterator(HW_EVENT_ID);
                setInterval(function () {
                    pubsub.publish(HW_EVENT_ID, {
                        helloWorldSubscription: {
                            msg: "Hello, World - " + counter++
                        }
                    });
                }, 200);
                return it;
            }
        }
    }
};
exports.default = resolver;
//# sourceMappingURL=hello-world-resolver.js.map