"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var body_parser_1 = __importDefault(require("body-parser"));
var express_1 = __importDefault(require("express"));
var graphql_1 = require("./graphql");
var context_1 = require("./graphql/context");
var http_1 = require("http");
var dotenv_1 = __importDefault(require("dotenv"));
var cors_1 = __importDefault(require("cors"));
var pre_fetcher_1 = require("./pre-fetcher");
var account_address_loader_1 = __importDefault(require("./utilities/account-address-loader"));
dotenv_1.default.config();
var PORT = process.env.PORT;
var app = express_1.default();
app.use(body_parser_1.default.json());
app.use(express_1.default.urlencoded({ extended: true }));
app.use(cors_1.default());
/**
 * GraphQL endpoint
 */
console.log(process.env.NODE_ENV);
app.post('/graphql', function (req, res) {
    var authHeader = req.header('authorization');
    var context = undefined;
    if (process.env.NODE_ENV == 'production') {
        if (authHeader) {
            var tokenResult = context_1.Context.verifyToken(authHeader);
            context = !tokenResult ? undefined : new context_1.Context(tokenResult);
        }
    }
    else {
        account_address_loader_1.default().then(function (val) { return (context = new context_1.Context(val)); });
    }
    console.log(context);
    graphql_1.serveGraphQLRequest({
        source: req.body.query,
        operationName: req.body.operationName,
        variableValues: req.body.variables,
        contextValue: context
    }, res);
});
//end of graphql setup
exports.server = http_1.createServer(app);
exports.server.listen(PORT, function () {
    graphql_1.createGraphQLSubscription(exports.server);
    console.log("Express server initialized on port " + PORT);
    console.log("GraphQL requests are enabled on /graphql endpoint via POST requests");
    if (!process.env.CI) {
        pre_fetcher_1.populateCache();
    }
});
//# sourceMappingURL=server.js.map