"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var web3_1 = __importDefault(require("web3"));
var contract_address_loader_1 = __importDefault(require("../utilities/contract-address-loader"));
var dotenv_1 = __importDefault(require("dotenv"));
var memory_cache_1 = __importDefault(require("memory-cache"));
var global_descriptor_resolvers_1 = require("../graphql/resolvers/global-descriptor-resolvers");
dotenv_1.default.config();
var TRANSACTION_TYPE;
(function (TRANSACTION_TYPE) {
    TRANSACTION_TYPE[TRANSACTION_TYPE["TRANSACTION_HASH"] = 0] = "TRANSACTION_HASH";
    TRANSACTION_TYPE[TRANSACTION_TYPE["RECIEPT"] = 1] = "RECIEPT";
    TRANSACTION_TYPE[TRANSACTION_TYPE["CONFIRMATION"] = 2] = "CONFIRMATION";
    TRANSACTION_TYPE[TRANSACTION_TYPE["ERROR"] = 3] = "ERROR";
})(TRANSACTION_TYPE || (TRANSACTION_TYPE = {}));
/**
 * This class's purpose is to be an abstraction for interacting with the Blockchain, and in particular,
 * the GlobalDescriptor Smart Contract. This class implements methods that are in the smart contract and executes
 * it on behalf of the invoker of this class. The actual smart contract can be found in ./contracts/GlobalDescriptor.sol
 *
 * We want to create a class like this for every smart contract.
 */
var GlobalDescriptorService = /** @class */ (function () {
    /**
     * Initializes the Web3 client that connects to the blockchain and creates the contract object.
     */
    function GlobalDescriptorService(web3Client, contract) {
        if (web3Client === void 0) { web3Client = new web3_1.default(process.env.BLOCKCHAIN_URL); }
        this.DECIMAL_OFFSET = Math.pow(10, 4);
        this.web3Client = web3Client;
        this.contract =
            contract == undefined
                ? new this.web3Client.eth.Contract(this.getContractAbi(), contract_address_loader_1.default('GlobalDescriptor'))
                : contract;
    }
    GlobalDescriptorService.BNToNumber = function (val) {
        return parseFloat(val);
    };
    /**
     * Abstraction for GlobalDescriptor (smart contract) method, insertValue(unit: string, value: number)
     * See ./contracts/GlobalDescriptor.sol for the actual contract method
     * @param accountId ID of the account sending the request (Local blockchain autogenerates 10 accounts to use)
     * @param value Data needed to enter itno blockchain
     * @param gas Optional paramter, defaults to 5,000,000. Need gas to perform any sort of operation.
     */
    GlobalDescriptorService.prototype.insertValue = function (accountId, value, gas) {
        var _this = this;
        if (gas === void 0) { gas = 5000000; }
        return new Promise(function (resolve, reject) {
            if (value.latitude == null || value.longitude == null) {
                value.latitude = 999;
                value.longitude = 999;
            }
            var transaction = _this.contract.methods.insertValue(value.unit, Math.floor(value.value * _this.DECIMAL_OFFSET), Math.floor(value.longitude * _this.DECIMAL_OFFSET), Math.floor(value.latitude * _this.DECIMAL_OFFSET));
            var txOptions = {
                from: accountId,
                gas: gas
            };
            transaction
                .send(txOptions)
                .on('transactionHash', function (txHash) {
                resolve(txHash);
            })
                .on('receipt', function () {
                var arr = memory_cache_1.default.get(global_descriptor_resolvers_1.GLOBAL_CACHE_KEY + value.unit);
                if (arr != null && Array.isArray(arr)) {
                    arr.push(__assign(__assign({}, value), { unixTimestamp: Date.now() }));
                    memory_cache_1.default.put(global_descriptor_resolvers_1.GLOBAL_CACHE_KEY + value.unit, arr);
                }
            })
                .on('error', function (err) { return reject(err.message); });
        });
    };
    /**
     * Abstraction for GlobalDescriptors (smart contract) method, getLatestValueForUnit(unit: string): number
     * See ./contracts/GlobalDescriptors.sol for the actual contract method
     * @param accountId ID of the account sending the request (Local blockchain autogenerates 10 accounts to use)
     * @param unit A unit such as lb, cm, miles, kilometer, etc
     * @param gas Optional paramter, defaults to 5,000,000. Need gas to perform any sort of operation.
     */
    GlobalDescriptorService.prototype.getLatestValueForUnit = function (accountId, unit, gas) {
        if (gas === void 0) { gas = 5000000; }
        return __awaiter(this, void 0, void 0, function () {
            var txOptions, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        txOptions = {
                            from: accountId,
                            gas: gas
                        };
                        return [4 /*yield*/, this.contract.methods
                                .getLatestUnitValue(unit)
                                .call(txOptions)];
                    case 1:
                        value = (_a.sent());
                        return [2 /*return*/, parseInt(value, 10) / this.DECIMAL_OFFSET];
                }
            });
        });
    };
    /**
     * Abstraction for GlobalDescriptors (smart contract) method, getAllAvailableUnitForGlobal(): string[]
     * See ./contracts/GlobalDescriptors.sol for the actual contract method
     * @param accountId ID of the account sending the request (Local blockchain autogenerates 10 accounts to use)
     * @param gas Optional paramter, defaults to 5,000,000. Need gas to perform any sort of operation.
     */
    GlobalDescriptorService.prototype.getAllAvailableUnitsForGlobal = function (accountId, gas) {
        if (gas === void 0) { gas = 5000000; }
        return __awaiter(this, void 0, void 0, function () {
            var txOptions;
            return __generator(this, function (_a) {
                txOptions = {
                    from: accountId,
                    gas: gas
                };
                return [2 /*return*/, this.contract.methods.getAllAvailableUnits().call(txOptions)];
            });
        });
    };
    /**
     * Abstraction for GlobalDescriptors (smart contract) method, getAllValuesRecordedForUnit(unit: string): number[]
     * See ./contracts/GlobalDescriptors.sol for the actual contract method
     * @param accountId ID of the account sending the request (Local blockchain autogenerates 10 accounts to use)
     * @param unit A unit such as lb, cm, miles, kilometer, etc
     * @param gas Optional paramter, defaults to 5,000,000. Need gas to perform any sort of operation.
     */
    GlobalDescriptorService.prototype.getAllValuesRecordedForUnit = function (accountId, unit, gas) {
        if (gas === void 0) { gas = 5000000; }
        return __awaiter(this, void 0, void 0, function () {
            var array, txOptions, stringArray;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        array = memory_cache_1.default.get(global_descriptor_resolvers_1.GLOBAL_CACHE_KEY + unit);
                        if (array) {
                            return [2 /*return*/, array];
                        }
                        txOptions = {
                            from: accountId,
                            gas: gas
                        };
                        return [4 /*yield*/, this.contract.methods
                                .getAllUnitValues(unit)
                                .call(txOptions)];
                    case 1:
                        stringArray = _a.sent();
                        return [2 /*return*/, stringArray.map(function (val) {
                                return {
                                    unit: unit,
                                    value: GlobalDescriptorService.BNToNumber(val.unitValue) /
                                        _this.DECIMAL_OFFSET,
                                    longitude: GlobalDescriptorService.BNToNumber(val.longitude) /
                                        _this.DECIMAL_OFFSET,
                                    latitude: GlobalDescriptorService.BNToNumber(val.latitude) /
                                        _this.DECIMAL_OFFSET,
                                    unixTimestamp: GlobalDescriptorService.BNToNumber(val.time)
                                };
                            })];
                }
            });
        });
    };
    GlobalDescriptorService.prototype.getPaginatedValuesRecordedForUnit = function (accountId, unit, start, count, gas) {
        if (start === void 0) { start = 0; }
        if (count === void 0) { count = 50; }
        if (gas === void 0) { gas = 5000000; }
        return __awaiter(this, void 0, void 0, function () {
            var array, stringArray, retArr;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        array = memory_cache_1.default.get(global_descriptor_resolvers_1.GLOBAL_CACHE_KEY + unit);
                        if (array && array.length >= count) {
                            return [2 /*return*/, array.slice(array.length - count, array.length)];
                        }
                        return [4 /*yield*/, this.contract.methods
                                .getPaginatedUnitValues(unit, start, count)
                                .call({ from: accountId, gas: gas })];
                    case 1:
                        stringArray = _a.sent();
                        retArr = stringArray.map(function (val) {
                            return {
                                unit: unit,
                                value: GlobalDescriptorService.BNToNumber(val.unitValue) /
                                    _this.DECIMAL_OFFSET,
                                longitude: GlobalDescriptorService.BNToNumber(val.longitude) /
                                    _this.DECIMAL_OFFSET,
                                latitude: GlobalDescriptorService.BNToNumber(val.latitude) /
                                    _this.DECIMAL_OFFSET,
                                unixTimestamp: GlobalDescriptorService.BNToNumber(val.time)
                            };
                        });
                        memory_cache_1.default.put(global_descriptor_resolvers_1.GLOBAL_CACHE_KEY + unit, retArr);
                        return [2 /*return*/, retArr];
                }
            });
        });
    };
    GlobalDescriptorService.prototype.insertValueAsync = function (accountId, value, subscriptionCallback, gas) {
        var _this = this;
        if (gas === void 0) { gas = 5000000; }
        return new Promise(function (resolve, reject) {
            if (value.latitude == null || value.longitude == null) {
                value.latitude = 999;
                value.longitude = 999;
            }
            var transaction = _this.contract.methods.insertValue(value.unit, Math.floor(value.value * _this.DECIMAL_OFFSET), Math.floor(value.longitude * _this.DECIMAL_OFFSET), Math.floor(value.latitude * _this.DECIMAL_OFFSET));
            var txOptions = {
                from: accountId,
                gas: gas
            };
            transaction
                .send(txOptions)
                .on('transactionHash', function (rec) {
                resolve(rec);
            })
                .on('receipt', function (rec) {
                subscriptionCallback(rec.transactionHash, TRANSACTION_TYPE.RECIEPT, rec.blockHash);
            })
                .on('confirmation', function (conf, rec) {
                subscriptionCallback(rec.transactionHash, TRANSACTION_TYPE.CONFIRMATION, conf.toString(10));
            })
                .on('error', function (err) {
                reject(err.message);
            });
        });
    };
    /**
     * This allows Web3JS to add methods to a contract object that we can invoke for our using the
     * JSON ABI (Application Binary Interface). The JSON ABI has the contract methods that can be
     * invoked with input paramters/types, and output paramters/types.
     */
    GlobalDescriptorService.prototype.getContractAbi = function () {
        var rawJson = fs_1.default.readFileSync(GlobalDescriptorService.GLOBAL_DESCRIPTOR_ABI_PATH);
        return JSON.parse(rawJson.toString()).abi;
    };
    /**
     * GLOBAL_DESCRIPTOR_ABI_PATH is the path to the JSON that has information about the SmartContract that
     * web3js can read. This allows Web3JS to add methods to a contract object that we can invoke from JavaScript.
     */
    GlobalDescriptorService.GLOBAL_DESCRIPTOR_ABI_PATH = 'build/contracts/GlobalDescriptor.json';
    return GlobalDescriptorService;
}());
exports.GlobalDescriptorService = GlobalDescriptorService;
//# sourceMappingURL=global-descriptor-service.js.map